{$V-,X+,S-}

Unit
   WinTools;

Interface

Uses
   Objects, OWindows, OStdDlgs;

Type
   SymbolSet = Set Of Char;
   TBuffer = Array[0..1024] of Char;
   TDateStr = String[10];

Type
  PTextStream = ^TTextStream;
  TTextStream = Object(TBufStream)
    Procedure WriteStr(P: PString);
    Function  ReadStr: PString;
  End;

  PLogFile = ^TLogFile;
  TLogFile = Object(TTextStream)
    Procedure WriteTo(Const S: String); Virtual;
    Function ReadFrom: String;
  End;

  PNoSortedStringCollection = ^TNoSortedStringCollection;
  TNoSortedStringCollection = Object(TStringCollection)
    Function Compare(Key1, Key2: Pointer): Integer; virtual;
  End;

  PCenterInputDialog = ^TCenterInputDialog;
  TCenterInputDialog = Object(TInputDialog)
    Procedure SetupWindow; Virtual;
  End;

Procedure DisposesStr(var P: PString);
Procedure Disposes(var P: PObject);
Function LongCopy(Source: TBuffer; Pos: Word; Count: Byte): String;
Function CopyFile(SourceName, TargetName: PChar): LongInt;
Function DelTrash(S: String): String;
Function CheckInLousy(S: String; Symb: SymbolSet): String;
Function DelSpace(Source: String): String;
Function NormalDate(Const D: TDateStr): TDateStr;
Function FormatDate(S: String): String;
Function StrToPChar(S: String): PChar;
Function Exist(FileName:String):   Boolean;
Function UpperCase(S: String): String;
Function StrDate(D:String): String;
Function Replicate(A: Char; N: Byte): String;
Function BreakSum(R: String; Symb:Char): String;
Function Space(N: Byte): String;
Function NowFullDate: String;
Function StrSum(Inp: String):String;
Procedure BreakStr(Var S1, S2, S3: String; Len: Byte);
Function NormalStr(S: String): String;
Function UnNormalStr(S: String): String;
Procedure PrintFile(FName: String);
Function CenterStr(S: String; Len: Byte): String;
Function JustifyRight(S: String; Len: Byte): String;
Function InputBox(AParent: PWindowsObject; ACaption, APrompt: PChar;
                  var ADefText: String; ATextSize: Integer): Integer;
Function CheckDate(Const S: String): Boolean;
Procedure WritePChar(var T: Text; P: PChar);
Function GetIniName(FName, Ext: String): String;
Function OEMToANSIFile(InFile, OutFile: String): Boolean;

Implementation

Uses
  Strings, WinDos, WinProcs, WinPrn, WinTypes;


Procedure TCenterInputDialog.SetupWindow;
Begin
  Inherited SetupWindow;
  SetWindowPos(HWindow, 0, 160, 150, 0, 0,
               swp_NoSize or swp_NoZOrder or swp_NoActivate or swp_NoRedraw);
End;

Function NormalDate(Const D: TDateStr): TDateStr;
Begin
  NormalDate := Copy(D, 5, Length(D) - 4) + Copy(D, 3, 2) + Copy(D, 1, 2);
End;

Function FormatDate(S: String): String;
Var
  P: String[2];
  K: String[10];
  i: Byte;
  A: String;
Begin
  K := '';
  A := S;
  if A = '' then
  Begin
     FormatDate := '';
     Exit;
  End;
  P := Copy(A, 1, Pos('/',A) - 1);
  Delete(A, 1, Pos('/', A));
  While Length(P) < 2 do
    P := '0' + P;
  K := P + '/';
  P := Copy(A, 1, Pos('/',A) - 1);
  Delete(A, 1, Pos('/', A));
  While Length(P) < 2 do
    P := '0' + P;
  While Length(A) < 2 do
     A := '0' + A;
  if Length(A) = 2 then
  {  2000   !!!! }
     A := '19' + A;
  K := K + P + '/' + A;
  FormatDate := K;
End;

Function LongCopy(Source: TBuffer; Pos: Word; Count: Byte): String;
Var
  i: Word;
  E: String;
Begin
  E := '';
  for i := Pos to Pos + Count - 1 do
    E := E + Source[i];
  LongCopy := E;
End;

Function CopyFile(SourceName, TargetName: PChar): LongInt;
{ Function return count of copying bytes,
  or 0 if copying not successful. }
Var
   Source, Target: PBufStream;
Begin
     CopyFile := 0;
     Source := New(PBufStream, Init(SourceName, stOpenRead, 2048));
     if Source^.Status <> stOk then
     Begin
        Dispose(Source, Done);
        Exit
     end;

     Target := New(PBufStream, Init(TargetName, stCreate, 2048));
     Target^.CopyFrom(Source^, Source^.GetSize);

     if Source^.GetSize <> Target^.GetSize then
        CopyFile := 0
     else
        CopyFile := Target^.GetSize;

     Dispose(Source, Done);
     Dispose(Target, Done);
End;

{ TTextStream Object Application}
Function  TTextStream.ReadStr: PString;
var
  S: String[255];
  i: Byte;
begin
  for i := 1 to 255 do begin
    Read(S[i], 1);
    if S[i] = #10 then
       Break;
    if (Status <> stOk) then
      if i <> 1 then
        Break
      else
      Begin
        ReadStr := Nil;
        Exit;
      End;
  end;
  S[0] := Char(i-2);
  ReadStr := NewStr(S);
end;

Procedure TTextStream.WriteStr(P: PString);
begin
  Write(P^[1], Byte(P^[0]));
end;

Procedure TLogFile.WriteTo;
Var
  P: PString;
Begin
  P := NewStr(S);
  WriteStr(P);
  DisposeStr(P);
  Flush;
End;

Function TLogFile.ReadFrom: String;
Var
  P: PString;
Begin
  P := ReadStr;
  if Assigned(P) then
     ReadFrom := P^
  else
     ReadFrom := '';
  DisposesStr(P);
End;

Procedure DisposesStr(var P: PString);
Begin
  if Assigned(P) then
  begin
    DisposeStr(P);
    P := Nil;
  End;
End;

Procedure Disposes(var P: PObject);
Begin
  if Assigned(P) then
  Begin
    Dispose(P, Done);
    P := Nil
  End
End;

Function DelTrash(S: String): String;
Begin
  DelTrash := CheckInLousy(S, ['0'..'9', '.']);
End;

Function CheckInLousy(S: String; Symb: SymbolSet): String;
Var
  i: Byte;
  P: String;
Begin
  P := '';
  For i := 1 to Length(S) do
  if Char(S[i]) in Symb then
     P := P + S[i];
  CheckInLousy := P
End;

Function TNoSortedStringCollection.Compare(Key1, Key2: Pointer): Integer;
Begin
  Compare := -1;
End;

Function StrToPChar(S: String): PChar;
Var
  P: Array[0..80] of Char;
Begin
  StrPCopy(P, S);
  StrToPChar := P
End;

Function DelSpace(Source: String): String;
Var
  S: String;
Begin
  S := Source;
  While (S[1] = ' ') and (S <> '') do
    Delete(S, 1, 1);
  While (S[Byte(S[0])] = ' ') and (S <> '') do
    Dec(S[0]);
  DelSpace := S;
End;

function Exist(FileName:String):Boolean;
var
   fil : file;
begin
  assign(fil,FileName);
  {$I-}
  reset(fil);
  {$i+}
  if IOresult <> 0 then
    Exist:=False
  else
  begin
    close(fil);
    Exist:=True;
  end;
end;

Function UpperCase(S: String): String;
Var
  A: Array[0..79] of Char;
  P: PChar;
Begin
  StrPCopy(A, S);
  P := StrUpper(A);
  UpperCase := StrPas(P);
End;

Function StrDate(D:String): String;
Const
     Month : Array[1..12] Of String[9] =
      (' ',' ',' ',' ',' ',' ',
       ' ',' ',' ',' ',' ',' ');
Var
   S, Day, Mon, Year: String;
   k,i: Byte;
   Code: Integer;
Begin
   if (Length(D) < 6) or (Pos('/', D) = 0)  then
   Begin
     StrDate := '  ';
     Exit;
   End;
   i := 1;
   Day := '';
   Mon := '';
   Year := '';
   While D[i] <> '/' do
   Begin
     Day := Day + D[i];
     Inc(i);
   End;
   Inc(i);
   While D[i] <> '/' do
   Begin
     Mon := Mon + D[i];
     Inc(i);
   End;
   Inc(i);
   While i < Length(D) + 1 do
   Begin
     Year := Year + D[i];
     Inc(i);
   End;
   Val(Mon,k,Code);
   if k in [1..12] then
     S := Concat(Day,' ', Month[k], Year,' .')
   else
     S := '  ';
   While Length(S) < 19 do
     S := S + ' ';
   StrDate := S;
End;

Function Replicate(A: Char; N: Byte): String;
Var
  i: Byte;
  S: String;
Begin
  S := '';
  For i := 1 to N do
      S := S + A;
  Replicate := S
end;

Function BreakSum(R: String; Symb:Char): String;
Var
  i, j: Byte;
  P, K, S: String;
Begin
{ S := DelTrash(R); }
  S := R;
  P := '';
  j := 1;
  For i := Length(S) downto 1 do
    P := P + S[i];
  i := 1;
  K := '';
  if Pos('.', P) <> 0 then
  Repeat
    K := K + P[i];
    Inc(i);
  Until P[i-1] = '.';

  j := 0;

  While i <= Length(P) do
  Begin
    if j = 3 then
    Begin
      K := K + Symb;
      j := 0;
    End;
    K := K + P[i];
    Inc(j);
    Inc(i);
  End;
  j := 1;
  P := '';
  For i := Length(K) downto 1 do
    P := P + K[i];
  BreakSum := P;
End;

Function Space(N: Byte): String;
Begin
  Space := Replicate(' ', N);
End;

Function NowFullDate: String;
var
  y, m, d, dow : Word;
  S,P: String;
begin
  GetDate(y,m,d,dow);
  Str(D:0,P);
  S := P + '/';
  Str(m:0,P);
  S := S + P + '/';
  Str(Y:0, P);
  NowfullDate := S + P;
end;

Function StrSum(Inp: String):String;
Const
  CifMan: Array[1..19] of String[12] =
       ('', '', '', '', '',
        '', '', '', '', '',
        '', '', '',
        '', 'a', '',
        '', '', '');
  DecCif: Array[2..9] of String[11] =
        ('', '', '', '',
         '', '', '', '');
  StoCif: Array[1..9] of String[9] =
        ('', '', '', '', '',
         '', '', '', '');

  Procedure UpChar(Var C: Char);
  Begin
    Case C of
      '': C := '';
      '': C := '';
      '': C := '';
      '': C := '';
      '': C := '';
      '': C := '';
      '': C := '';
      '': C := '';
    End;
  End;

Var
  S,P,Cop       : String;
  Out           : Boolean;
  Ch,Chh        : Char;
  Code, C       : Integer;
  i,j,k: 1..4;  { 1 -  1000;
              2 -  .;
              3 -  .;
              4 -  . }
Begin
  S := Inp;
  While S[1] = '0' do
     Delete(S, 1, 1);

  if Pos('.',S) <> 0 then
  Begin
    Cop := Copy(S, Pos('.', S)+1, 2);
    Delete(S, Pos('.', S), 10);
  End
  else
     Cop := '';

  While Length(Cop) <> 2 do
     Cop := '0' + Cop;

  Cop := Cop + ' .';


  i := 1;
  Out := True;
  While Out do
  Begin
    if Length(S) > 3 then
    Begin
       P := Copy(S, Length(S) - 2, 3);
       Delete(S, Length(S) - 2, 3);
    End
    else
    Begin
      P := S;
      Out := False;
    End;

    Ch := Char(P[Length(P)]);
    Chh := Char(P[Length(P)-1]);

    Case i of
      1: if Chh = '1' then
            Cop := ' ' + Cop
         else
         Begin
           Case Ch of
             #0 : Begin
                    StrSum := '00  ' + Cop;
                    Exit;
                  End;
             '1': Cop := ' ' + Cop;
             '2'..'4': Cop := ' ' + Cop;
           else
             Cop := ' ' + Cop;
           End;
         End;
      2: if P <> '000' then
         if Chh = '1' then
           Cop := ' ' + Cop
         else
           Case Ch of
             '1': Cop := ' ' + Cop;
             '2'..'4': Cop := ' ' + Cop;
           else
             Cop := ' ' + Cop;
           End;
      3: if P <> '000' then
         if Chh = '1' then
            Cop := ' ' + Cop
         else
         Case Ch of
           '1': Cop := ' ' + Cop;
           '2'..'4': Cop := ' ' + Cop;
         else
           Cop := ' ' + Cop;
         End;
      4: if Chh = '1' then
            Cop := ' ' + Cop
         else
         Case Ch of
           '1': Cop := ' ' + Cop;
           '2'..'4': Cop := ' ' + Cop;
           else
             Cop := ' ' + Cop;
         End;
    End;

    k := 1;
    While (P[1] = '0') and (P <> '') do
      Delete(P, 1, 1);

    While True do
    Begin
      if P = '' then
         Break;
      j := Length(P);
{      Dec(P[0]); }
      Ch := Char(P[j]);
      Val(Ch, C, Code);

      Case k of
        1: if Chh = '1' then
              Cop := CifMan[10+C] + ' ' + Cop
           else
           if C <> 0 then
           Begin
             if i = 2 then
                Case Ch of
                   '1': Cop := ' ' + Cop;
                   '2': Cop := ' ' + Cop;
                else
                   Cop := CifMan[C] + ' ' + Cop;
                End
             else
                Cop := CifMan[C] + ' ' + Cop;
             Chh := Ch;
           End;
        2: if (C <> 0) and (C <> 1) then
              Cop := DecCif[C] + ' ' + Cop;
        3: Cop := StoCif[C] + ' ' + Cop;
      End;
      Inc(k);
      Dec(P[0]);
    End;
    Inc(i);
  End;
  P := Cop;
  While Pos('  ', P) <> 0 do
    Delete(P, Pos('  ', P), 1);
  Cop := P;
{  Cop[1] := UpCase(Cop[1]);}
  UpChar(Cop[1]);
  StrSum := Cop;
End;

Procedure BreakStr(Var S1, S2, S3: String; Len: Byte);
Var
  i, j : Byte;
  P    : String;
  Out  : Boolean;
  S    : Array[1..4] of String;
Begin
  If Length(S1) < Len then Exit;
  P := '';
  S[1] := S1;
  S[2] := S2;
  S[3] := S3;
  Out := True;
  j := 1;
  While Out do
  Begin
    While True do
    Begin
      i := Pos(' ', S[j]);
      if Length(P) + i > Len then Break;
      if i = 0 then
      Begin
        if Length(P) + Length(S[j]) > Len then Break;
        P := P + S[j];
        S[j] := '';
        Break;
      End;
      P := P + Copy(S[j], 1, i);
      Delete(S[j], 1, i);
    End;
    S[j+1] := S[j];
    S[j] := P;
    P := '';
    if j = 3 then Break;
    Inc(j);
  End;
  S1 := S[1];
  S2 := S[2];
  S3 := S[3];
End;


Function NormalStr(S: String): String;
Var
  P: Array[0..254] of Char;
  J: String;
Begin
    StrPCopy(P, S);
    ANSIToOEM(P, P);
    J := StrPas(P);
    NormalStr := J;
End;

Function UnNormalStr(S: String): String;
Var
  P: Array[0..254] of Char;
  J: String;
Begin
  StrPCopy(P, S);
  OEMToANSI(P, P);
  J := StrPas(P);
  UnNormalStr := J;
End;

Procedure PrintFile;
Var
  PRFile, PRinter: TExt;
  S: String;
Begin
  Assign(PRinter, 'PRN');
  ReWRite(Printer);
  Assign(PrFile, FName);
  Reset(PrFile);
  While not eof( prFile) do
  Begin
    readln(prfile, s);
    writeln(printer, s);
  End;
  close(printer);
  close(Prfile);
End;

Function CenterStr(S: String; Len: Byte): String;
Var
  P: String;
Begin
  if Len < Length(S) then
     CenterStr := Copy(S, 1, Len)
  else
  Begin
    P := Space((Len - Length(S)) shr 1) + S;
    While Length(P) <> Len do
      P := P + ' ';
    CenterStr := P;
  End;
End;

Function JustifyRight(S: String; Len: Byte): String;
Begin
  JustifyRight := S;
  if Length(S) < Len then
     JustifyRight := Space((Len - Length(S))) + S;
End;

Function InputBox(AParent: PWindowsObject; ACaption, APrompt: PChar;
                  var ADefText: String; ATextSize: Integer): Integer;
Var
  D: PInputDialog;
  A: Array[0..80] of Char;
Begin
  StrPCopy(A, ADefText);
  D := New(PCenterInputDialog, Init(AParent, ACaption, APrompt, A, ATextSize));
  D^.TransferBuffer := @A;
  InputBox := Application^.ExecDialog(D);
  ADefText := StrPas(A)
End;

Function CheckDate(Const S: String): Boolean;
Var
  P: String[2];
  i: Byte;
  A: String;
Begin
  A := FormatDate(S);
  CheckDate := True;
  if A = '' then
     Exit;
  P := Copy(A, 1, Pos('/',A) - 1);
  Delete(A, 1, Pos('/', A));
  While Length(P) < 2 do
    P := '0' + P;
  if (P < '01') or (P > '31') then
  Begin
    CheckDate := False;
    Exit;
  End;
  P := Copy(A, 1, Pos('/',A) - 1);
  Delete(A, 1, Pos('/', A));
  While Length(P) < 2 do
    P := '0' + P;
  if (P < '01') or (P > '12') then
  Begin
    CheckDate := False;
    Exit;
  End;
End;

Procedure WritePChar(var T: Text; P: PChar);
Var
  i: Word;
Begin
  for i := 1 to StrLen(P) do
      Write(T, P[i])
End;

Function  ReadPChar(var T: Text): PChar;
var
  S: Array[0..1023] of Char;
  i: Word;
begin
  StrCopy(S, '');
  for i := 0 to 1023 do
  begin
    {$I-}
    Read(T, S[i]);
    if IOResult <> 0 then
       Break;
    if S[i] = #10 then
       Break;
  end;
  S[i-1] := #0;
  ReadPChar := S;
end;

Function GetIniName(FName, Ext: String): String;
Begin
  if Pos('.', FName) <> 0 then
     GetIniName := Copy(FName, 1, Pos('.', FName)) + Ext
  else
     GetIniName := FName + Ext
End;

Function OEMToANSIFile(InFile, OutFile: String): Boolean;
Var
  S, P: String;
  A, B: Text;
  J: Array[0..1023] of Char;
Begin
  OEMToANSIFile := False;
  if Not Exist(InFile) then
     Exit;
  Assign(A, InFile);
  {$I-}
  Reset(A);
  if IOResult <> 0 then
     Exit;
  {$I+}
  Assign(B, OutFile);
  {$I-}
  ReWrite(B);
  if IOResult <> 0 then
  Begin
    Close(A);
    Exit;
  End;
  {$I+}
  While Not Eof(A) do
  Begin
    ReadLn(A, S);
    P := S;
    P := UnNormalStr(P);
    WriteLn(B, P);
{    J := ReadPChar(A);
    OEMToANSI(J, J);
    WritePChar(B, J);
    WriteLn(B); }
  End;
  Close(A);
  Close(B);
  OEMToANSIFile := True;
End;

End.
